from matrix import *

class Graph:
    """
    represents a graph with N nodes [0...(N-1)].
    Fields:
    edges - holds an N-by-N adjacency matrix of edges, where each
            edge has a positive weight.
    nodes - a list of node weights
    """

    def __init__(self, n, nodeVal=0, edgeVal=0):
        """
        create a new matrix of edgeVal's and a list of nodeVal's
        """
        assert n > 0
        self.nodes = [nodeVal]*n
        self.edges = Matrix(n,n,edgeVal)

    def __repr__(self):
        return "<Graph, nodes:{0}, edges:{1}>".format(self.nodes,self.edges)

    def dim(self):
        return len(self.nodes)

    @staticmethod
    def load(nodefile,edgefile):
        # read nodes
        f = open(nodefile)
        n = int(f.readline())
        line = f.readline()
        nodes = [int(x) for x in line.split()]
        assert len(nodes) == n
        f.close()
        # read edges
        edges = Matrix.load(edgefile)
        assert edges.dim() == (n,n)
        # attach nodes and edges to make a graph
        graph = Graph(1)
        graph.nodes = nodes
        graph.edges = edges
        return graph
